package com.baidu.cloud.videoplayer.demo;

import android.Manifest;
import android.annotation.TargetApi;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.content.ContentResolver;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.database.Cursor;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.MediaStore;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.view.Window;
import android.view.WindowManager;
import android.widget.BaseAdapter;
import android.widget.ImageButton;
import android.widget.ImageView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import com.baidu.cloud.media.download.DownloadableVideoItem;
import com.baidu.cloud.media.download.VideoDownloadManager;
import com.baidu.cloud.videoplayer.demo.info.DownloadObserverManager;
import com.baidu.cloud.videoplayer.demo.info.SampleObserver;
import com.baidu.cloud.videoplayer.demo.info.SharedPrefsStore;
import com.baidu.cloud.videoplayer.demo.info.VideoInfo;
import com.baidu.cloud.videoplayer.demo.popview.AddPopWindow;
import com.baidu.cloud.videoplayer.demo.popview.CustomAlertWindow;
import com.baidu.cloud.videoplayer.demo.popview.MorePopWindow;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;

public class SampleListActivity extends Activity {
    private static final String TAG = "SampleListActivity";
    private ListView listView;
    private ArrayList<VideoInfo> listData = new ArrayList<VideoInfo>();
    private boolean bForeground;
    private static final int REQUEST_STORAGE_ACCESS_PERMISSION = 101;

    public static final String SAMPLE_USER_NAME = "sampleUser";
    VideoDownloadManager downloadManagerInstance;

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        // 设置状态栏颜色
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            Window window = this.getWindow();
            window.clearFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS);
            window.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS);
            // finally change the color
            window.setStatusBarColor(0xfff7f7f7);
        }
        setContentView(R.layout.activity_main);

        downloadManagerInstance = VideoDownloadManager.getInstance(SampleListActivity.this, SampleListActivity.SAMPLE_USER_NAME);

        listView = (ListView) this.findViewById(R.id.lv_video_list);
        listView.setAdapter(adapter);
        initOtherUI();
    }

    private void initOtherUI() {
        ImageButton ibMore = (ImageButton) this.findViewById(R.id.ibtn_more);
        ibMore.setOnClickListener(new OnClickListener() {

            @Override
            public void onClick(View v) {
                MorePopWindow popWindow = MorePopWindow.createMorePopWindow(SampleListActivity.this);
                popWindow.showPopupWindow(findViewById(R.id.rl_top_bar));
            }

        });
    }

    @Override
    protected void onResume() {
        Log.d(TAG, "onResume");
        if (Build.VERSION.SDK_INT >= 23) {
            if (this.checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
                requestPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE, "读写设备上的照片和文件", REQUEST_STORAGE_ACCESS_PERMISSION);
            }
        }
        bForeground = true;
        refreshData();
        openDiscoverable();
        super.onResume();

    }

    public void refreshData() {
        listData = SharedPrefsStore.getAllMainVideoFromSP(this);
        adapter.notifyDataSetChanged();
    }

    private void openDiscoverable() {
        BluetoothAdapter adapter = BluetoothAdapter.getDefaultAdapter();
        if (adapter.getScanMode() != BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE) {
            Intent discoverableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE);
            discoverableIntent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 300);
            startActivity(discoverableIntent);
        }
    }

    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == AddPopWindow.REQUEST_CODE_SELECT_VIDEO) {
            if (resultCode == RESULT_OK) {
                Uri uri = data.getData();
                ContentResolver cr = this.getContentResolver();
                Cursor cursor = cr.query(uri, null, null, null, null);
                if (cursor != null) {
                    if (cursor.moveToFirst()) {
                        // 视频名称：MediaStore.Audio.Media.TITLE
                        if (cursor.getColumnIndex(MediaStore.Video.Media.TITLE) >= 0) {
                            String title = cursor.getString(cursor.getColumnIndexOrThrow(MediaStore.Video.Media.TITLE));
                            AddPopWindow.selectName = title;
                        }

                        // 视频路径：MediaStore.Audio.Media.DATA
                        if (cursor.getColumnIndex(MediaStore.Video.Media.DATA) >= 0) {
                            String videoPath = cursor.getString(cursor.getColumnIndexOrThrow(MediaStore.Video.Media.DATA));
                            AddPopWindow.selectPath = videoPath;
                        }
                    }
                    cursor.close();
                }
            }
            super.onActivityResult(requestCode, resultCode, data);
        }
    }

    /**
     * 注：以下adapter仅为简单实现，实际项目中需要进行优化
     */
    private BaseAdapter adapter = new BaseAdapter() {

        private LayoutInflater mInflater;

        @Override
        public int getCount() {
            return listData.size();
        }

        @Override
        public Object getItem(int position) {
            return null;
        }

        @Override
        public long getItemId(int position) {
            return 0;
        }

        @Override
        public View getView(final int position, View convertView, ViewGroup parent) {
            if (mInflater == null) {
                mInflater = LayoutInflater.from(SampleListActivity.this);
            }
            if (convertView == null) {
                convertView = mInflater.inflate(R.layout.item_of_list_video, null);
            }
            final VideoInfo info = listData.get(position);
            TextView tvTitle = (TextView) convertView.findViewById(R.id.tv_item_title);
            TextView tvDesc = (TextView) convertView.findViewById(R.id.tv_item_desc);
            ImageView ivIcon = (ImageView) convertView.findViewById(R.id.iv_item_icon);
            ImageButton ibtnDelete = (ImageButton) convertView.findViewById(R.id.ibtn_item_delete);
            ImageButton ibtnDownload = (ImageButton) convertView.findViewById(R.id.ibtn_item_download);

            if (info.getImageUrl() != null && !info.getImageUrl().equals("")) {
                // fetch image from assets
                // if your image from url, you need to fetch image async
                InputStream ims;
                try {
                    ims = getAssets().open(info.getImageUrl());
                    Drawable iconDrawable = Drawable.createFromStream(ims, null);
                    ivIcon.setImageDrawable(iconDrawable);
                } catch (IOException e) {
                    e.printStackTrace();
                }

            } else {
                ivIcon.setImageResource(R.drawable.item_default_icon);
            }
            tvTitle.setText(info.getTitle());
            tvDesc.setText(info.getUrl());
            if (info.isCanDelete()) {
                ibtnDelete.setVisibility(View.VISIBLE);
                ibtnDelete.setOnClickListener(new OnClickListener() {

                    @Override
                    public void onClick(View v) {
                        String msgString = "确定要删除<" + info.getTitle() + ">这个视频资源嘛？";
                        String yesString = "确定";
                        String noString = "取消";
                        View.OnClickListener yesListener = new View.OnClickListener() {

                            @Override
                            public void onClick(View v) {
                                SharedPrefsStore.deleteMainVideo(SampleListActivity.this, info);
                                refreshData();
                            }
                        };
                        CustomAlertWindow.showAlertWindow(SampleListActivity.this, msgString, yesString, noString,
                                yesListener, null);
                    }

                });
            } else {
                ibtnDelete.setVisibility(View.INVISIBLE);
            }

            convertView.setOnClickListener(new OnClickListener() {

                @Override
                public void onClick(View v) {
                    if (!bForeground) {
                        return;
                    }
                    Intent intent = null;
                    int barType = SharedPrefsStore.getIntData(SampleListActivity.this,
                            C.KEY_CONTROLBAR_TYPE, C.BARTYPE_ADVANCE);
                    if (barType == C.BARTYPE_SIMPLE) {
                        // SimplePlayActivity简易播放窗口，便于快速了解播放流程
                        intent = new Intent(SampleListActivity.this, SimplePlayActivity.class);
                    } else {
                        // AdvancedPlayActivity高级播放窗口，内含丰富的播放控制逻辑
                        intent = new Intent(SampleListActivity.this, AdvancedPlayActivity.class);
                    }
                    intent.putExtra("videoInfo", info);
                    startActivity(intent);
                    bForeground = false;
                }

            });

            if (info.getUrl() != null && info.getUrl().endsWith(".m3u8")) {
                ibtnDownload.setVisibility(View.VISIBLE);
                ibtnDownload.setOnClickListener(new OnClickListener() {

                    @Override
                    public void onClick(View v) {
                        //
                        DownloadableVideoItem item = downloadManagerInstance
                                .getDownloadableVideoItemByUrl(info.getUrl());
                        if (item != null) {
                            // already
                            Toast.makeText(SampleListActivity.this, "该资源已经在缓存列表，请点击右上角「本地缓存」查看", Toast.LENGTH_SHORT).show();
                        } else {
                            SharedPrefsStore.addToCacheVideo(SampleListActivity.this, info);
                            SampleObserver sampleObs = new SampleObserver();
                            DownloadObserverManager.addNewObserver(info.getUrl(), sampleObs);
                            downloadManagerInstance.startOrResumeDownloader(info.getUrl(), sampleObs);
                            Toast.makeText(SampleListActivity.this, "开始缓存，请点击右上角「本地缓存」查看进度", Toast.LENGTH_SHORT).show();
                        }
                    }

                });
            } else {
                ibtnDownload.setVisibility(View.INVISIBLE);
            }
            return convertView;
        }

    };

    @TargetApi(23)
    private void requestPermission(final String permission, String rationale, final int requestCode) {
        if (this.shouldShowRequestPermissionRationale(permission)) {
            String msgString = "需要权限：" + rationale;
            String yesString = "确定";
            String noString = "取消";
            View.OnClickListener yesListener = new View.OnClickListener() {

                @Override
                public void onClick(View v) {
                    requestPermissions(new String[]{permission}, requestCode);
                }
            };
            CustomAlertWindow.showAlertWindow(SampleListActivity.this, msgString, yesString, noString,
                    yesListener, null);
        } else {
            requestPermissions(new String[]{permission}, requestCode);
        }
    }
}