package com.baidu.cloud.videoplayer.demo.info;

import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.content.Context;
import android.content.SharedPreferences;

import com.baidu.cloud.media.player.BDCloudMediaPlayer;

/**
 * 此处仅为存储示例 您的项目中可能使用其他存储形式
 *
 * @author baidu
 */
public class SharedPrefsStore {
    private static final String MAIN_VIDEO_LIST_SPNAME = "video-main";
    private static final String CACHE_VIDEO_LIST_SPNAME = "video-cache";
    private static final String SETTINGS_SPNAME = "video-settings";

    private static final String KEY_VIDEOS_ARRAY = "videos";

    public static boolean addToMainVideo(Context context, VideoInfo info) {
        SharedPreferences spList = context.getSharedPreferences(MAIN_VIDEO_LIST_SPNAME, Context.MODE_PRIVATE);
        String jsonStr = spList.getString(KEY_VIDEOS_ARRAY, "[]");
        try {
            JSONArray jsonArray = new JSONArray(jsonStr);
            jsonArray.put(info.toJson());
            SharedPreferences.Editor editor = spList.edit();
            editor.putString(KEY_VIDEOS_ARRAY, jsonArray.toString());
            editor.apply();
        } catch (JSONException e1) {
            e1.printStackTrace();
            return false;
        }

        return true;
    }

    public static boolean deleteMainVideo(Context context, VideoInfo info) {
        SharedPreferences spList = context.getSharedPreferences(MAIN_VIDEO_LIST_SPNAME, Context.MODE_PRIVATE);
        String jsonStr = spList.getString(KEY_VIDEOS_ARRAY, "[]");
        try {
            JSONArray jsonArray = new JSONArray(jsonStr);
            JSONArray newJsonArray = new JSONArray();
            for (int i = 0; i < jsonArray.length(); ++i) {
                try {
                    JSONObject json = (JSONObject) jsonArray.get(i);
                    VideoInfo info2 = VideoInfo.fromJson(json);
                    if (info2.getUrl().equals(info.getUrl()) && info2.getTitle().equals(info.getTitle())) {
                        continue;
                    } else {
                        newJsonArray.put(json);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            if (newJsonArray.length() == jsonArray.length()) {
                return false;
            } else {
                SharedPreferences.Editor editor = spList.edit();
                editor.putString(KEY_VIDEOS_ARRAY, newJsonArray.toString());
                editor.apply();
            }
        } catch (JSONException e1) {
            e1.printStackTrace();
        }
        return false;
    }

    public static boolean addToCacheVideo(Context context, VideoInfo info) {
        SharedPreferences spList = context.getSharedPreferences(CACHE_VIDEO_LIST_SPNAME, Context.MODE_PRIVATE);
        String jsonStr = spList.getString(KEY_VIDEOS_ARRAY, "[]");
        try {
            JSONArray jsonArray = new JSONArray(jsonStr);
            jsonArray.put(info.toJson());
            SharedPreferences.Editor editor = spList.edit();
            editor.putString(KEY_VIDEOS_ARRAY, jsonArray.toString());
            editor.apply();
        } catch (JSONException e1) {
            e1.printStackTrace();
            return false;
        }
        return true;
    }

    public static boolean deleteCacheVideo(Context context, VideoInfo info) {
        SharedPreferences spList = context.getSharedPreferences(CACHE_VIDEO_LIST_SPNAME, Context.MODE_PRIVATE);
        String jsonStr = spList.getString(KEY_VIDEOS_ARRAY, "[]");
        try {
            JSONArray jsonArray = new JSONArray(jsonStr);
            JSONArray newJsonArray = new JSONArray();
            for (int i = 0; i < jsonArray.length(); ++i) {
                try {
                    JSONObject json = (JSONObject) jsonArray.get(i);
                    VideoInfo info2 = VideoInfo.fromJson(json);
                    if (info2.getUrl().equals(info.getUrl()) && info2.getTitle().equals(info.getTitle())) {
                        continue;
                    } else {
                        newJsonArray.put(json);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            if (newJsonArray.length() == jsonArray.length()) {
                return false;
            } else {
                SharedPreferences.Editor editor = spList.edit();
                editor.putString(KEY_VIDEOS_ARRAY, newJsonArray.toString());
                editor.apply();
            }
        } catch (JSONException e1) {
            e1.printStackTrace();
        }
        return false;
    }

    public static ArrayList<VideoInfo> getAllMainVideoFromSP(Context context) {
        ArrayList<VideoInfo> infoList = new ArrayList<VideoInfo>();
        // add sample data first
        ArrayList<VideoInfo>  mainSampleList = getMainSampleData(context);
        for (VideoInfo info : mainSampleList) {
            if (info.getDemoEnable() > 0) {
                infoList.add(info);
            }
        }

        // add user added data
        SharedPreferences spList = context.getSharedPreferences(MAIN_VIDEO_LIST_SPNAME, Context.MODE_PRIVATE);
        String jsonStr = spList.getString(KEY_VIDEOS_ARRAY, "[]");
        try {
            JSONArray jsonArray = new JSONArray(jsonStr);
            for (int i = 0; i < jsonArray.length(); ++i) {
                try {
                    JSONObject json = (JSONObject) jsonArray.get(i);
                    VideoInfo info = VideoInfo.fromJson(json);
                    if (info.getDemoEnable() > 0) {
                        infoList.add(info);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        } catch (JSONException e1) {
            e1.printStackTrace();
        }

        return infoList;
    }

    /**
     * 初次进入应用，SP无数据时，准备样例数据
     *
     * @return
     */
    public static ArrayList<VideoInfo> getMainSampleData(Context context) {
        ArrayList<VideoInfo> sampleList = new ArrayList<VideoInfo>();

        String title1 = "百度云宣传视频";
        String url1 = "http://bdcoud-media-player.cdn.bcebos.com/demovideo/bdcloud.mp4";
        String[] preImages1 = {
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00001.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00002.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00003.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00004.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00005.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00006.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00007.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00008.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00009.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00010.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00011.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00012.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00013.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00014.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00015.jpg",
            "https://bdcloud-player.cdn.bcebos.com/demovideo/bdcloud_demo_sprite/mda-jaqf8wrtf9et9dwp.mp4-SPRITE-00016.jpg"
        };
        VideoInfo info1 = new VideoInfo(title1, url1);
        info1.setCanDelete(false);
        info1.setImageUrl("baidu_cloud_bigger.jpg");
        info1.setPreImages(preImages1);
        info1.setSwitchMode(BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE);
        sampleList.add(info1);

        defaultAddDemoSample(sampleList, "百度云智感超清",
                "https://bdcloud-player.cdn.bcebos.com/testvideo/subtitle/burn-in/mov_text/smartvision_soft_subtitle_3m.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList, "HLS片源",
                "https://bdcloud-player.cdn.bcebos.com/testvideo/hls/265/1080p/cctv/test_hls_h265_1080p_output.m3u8",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList, "播放Assets目录媒体资源",
                "asset://video_480x360_h264.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                false);

        defaultAddDemoSample(sampleList, "FLV片源",
                "https://bdcloud-player.cdn.bcebos.com/testvideo/flv/264/1080p/liulangdiqiu-264-1080.flv",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList, "MOV片源",
                "https://bdcloud-player.cdn.bcebos.com/testvideo/mov/265/1080p/liulangdiqiu-265-1080.mov",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList, "HLS多码率无缝切换",
                "https://bdcoud-media-player.cdn.bcebos.com/demovideo/all_hls/video.m3u8",
                BDCloudMediaPlayer.SOURCE_SWITCH_SMOOTH_HLS_MODE,
                true);

        defaultAddDemoSample(sampleList, "HLS多码率无缝切换2",
                "https://bdcloud-player.cdn.bcebos.com/testvideo/hls/265/adapt/test.m3u8",
                BDCloudMediaPlayer.SOURCE_SWITCH_SMOOTH_HLS_MODE,
                true);

        String title2 = "MP4多码率无缝切换";
        String url2 = "https://bdcloud-player.cdn.bcebos.com/testvideo/mp4/switchsource/sv1080p.mp4";
        String[] mediaItems =  new String[] {
                "https://bdcloud-player.cdn.bcebos.com/testvideo/mp4/switchsource/sv1080p.mp4",
                "https://bdcloud-player.cdn.bcebos.com/testvideo/mp4/switchsource/sv720p.mp4",
                "https://bdcloud-player.cdn.bcebos.com/testvideo/mp4/switchsource/sv480p.mp4"
        };
        String[] mediasDes =  new String[] {
                "1920x1080,2400000",
                "1280x720,1200000",
                "852x480,700000"
        };
        VideoInfo info2 = new VideoInfo(title2, url2);
        info2.setCanDelete(false);
        info2.setSwitchMode(BDCloudMediaPlayer.SOURCE_SWITCH_SMOOTH_MODE);
        info2.setInputList(mediaItems);
        info2.setMediasDes(mediasDes);
        info2.setDemoEnable(1);
        sampleList.add(info2);
        
        defaultAddDemoSample(sampleList, "完整版测试片源-MP4+字幕",
                "https://bdcloud-player.cdn.bcebos.com/testvideo/subtitle/burn-in/mov_text/Tos_1080_mov_text_3M.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList, "完整版测试片源-MKV+字幕",
                "https://bdcloud-player.cdn.bcebos.com/testvideo/subtitle/burn-in/srt/AVC_3_Ac_3_Srt.mkv",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);
        
        defaultAddDemoSample(sampleList, "完整版测试片源-AVI",
                "https://bdcloud-player.cdn.bcebos.com/testvideo/avi/264/720p/Big.avi",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList,
                "WANOS测试片源-阿凡达",
                "https://bdcloud-player.cdn.bcebos.com/demovideo/Avatar-WANOS.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList,
                "WANOS测试片源-Bailando",
                "https://bdcloud-player.cdn.bcebos.com/demovideo/Bailando-WANOS.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList,
                "WANOS测试片源-长津湖",
                "https://bdcloud-player.cdn.bcebos.com/demovideo/ChangJinHu-WANOS.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList,
                "WANOS测试片源-金刚川",
                "https://bdcloud-player.cdn.bcebos.com/demovideo/JinGangChuan-WANOS.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList,
                "WANOS测试片源-你好李焕英",
                "https://bdcloud-player.cdn.bcebos.com/demovideo/LiHuanYing-WANOS.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList,
                "HDR-Vivid-demo",
                "https://bdcloud-player.cdn.bcebos.com/demovideo/hdrdemo/vivid/SonyBraviaOLED4KDemo.vivid_bd265_4k.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList,
                "HDR-Vivid-demo2",
                "https://bdcloud-player.cdn.bcebos.com/demovideo/hdrdemo/vivid/4kHdrVividDemo.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList,
                "4K-HDR10-Colorful",
                "https://bdcloud-player.cdn.bcebos.com/demovideo/LG4KHDRDemo-Colorful-10m.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList,
                "4K-HDR10-Chess",
                "https://bdcloud-player.cdn.bcebos.com/demovideo/LG4KHDRDemo-Chess-10m.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList,
                "007-HDR10",
                "https://bdcloud-player.cdn.bcebos.com/demovideo/007_pq.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        defaultAddDemoSample(sampleList,
                "竖屏-HLG-ColorCard",
                "https://bdcloud-player.cdn.bcebos.com/demovideo/1080_1920_HLG_Color_Card.mp4",
                BDCloudMediaPlayer.SOURCE_SWITCH_NORMAL_MODE,
                true);

        return sampleList;
    }

    private static VideoInfo defaultAddDemoSample(ArrayList<VideoInfo> sampleList, String title, String url,
                                                  int switchMode, boolean demoEnable) {
        VideoInfo info = new VideoInfo(title, url);
        info.setCanDelete(false);
        info.setSwitchMode(switchMode);
        info.setDemoEnable(demoEnable ? 1 : 0);
        sampleList.add(info);
        return info;
    }

    public static ArrayList<VideoInfo> getAllCacheVideoFromSP(Context context) {
        ArrayList<VideoInfo> infoList = new ArrayList<VideoInfo>();
        SharedPreferences spList = context.getSharedPreferences(CACHE_VIDEO_LIST_SPNAME, Context.MODE_PRIVATE);
        String jsonStr = spList.getString(KEY_VIDEOS_ARRAY, "[]");
        try {
            JSONArray jsonArray = new JSONArray(jsonStr);
            for (int i = 0; i < jsonArray.length(); ++i) {
                try {
                    JSONObject json = (JSONObject) jsonArray.get(i);
                    VideoInfo info = VideoInfo.fromJson(json);
                    infoList.add(info);
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        } catch (JSONException e1) {
            e1.printStackTrace();
        }
        return infoList;
    }

    public static void saveData(Context context, String key, int value) {
        SharedPreferences spList = context.getSharedPreferences(SETTINGS_SPNAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = spList.edit();
        editor.putInt(key, value);
        editor.apply();
    }

    public static void saveData(Context context, String key, boolean value) {
        SharedPreferences spList = context.getSharedPreferences(SETTINGS_SPNAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = spList.edit();
        editor.putBoolean(key, value);
        editor.apply();
    }

    public static int getIntData(Context context, String key) {
        return getIntData(context, key, 0);
    }

    public static int getIntData(Context context, String key, int defValue) {
        SharedPreferences spList = context.getSharedPreferences(SETTINGS_SPNAME, Context.MODE_PRIVATE);
        return spList.getInt(key, defValue);
    }

    public static boolean getBooleanData(Context context, String key) {
        SharedPreferences spList = context.getSharedPreferences(SETTINGS_SPNAME, Context.MODE_PRIVATE);
        return spList.getBoolean(key, false);
    }

    public static void setPlayerFitMode(Context context, boolean isCrapping) {
        SharedPreferences spList = context.getSharedPreferences(SETTINGS_SPNAME, Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = spList.edit();
        editor.putBoolean("isCrapping", isCrapping);
        editor.apply();
    }

    public static boolean isPlayerFitModeCrapping(Context context) {
        SharedPreferences spList = context.getSharedPreferences(SETTINGS_SPNAME, Context.MODE_PRIVATE);
        return spList.getBoolean("isCrapping", false);
    }

    public static void setHeadTrack(Context context, boolean enable) {
        SharedPreferences spList = context.getSharedPreferences(SETTINGS_SPNAME, 0);
        SharedPreferences.Editor editor = spList.edit();
        editor.putBoolean("isHeadTrack", enable);
        editor.apply();
    }

    public static boolean isHeadTrack(Context context) {
        SharedPreferences spList = context.getSharedPreferences(SETTINGS_SPNAME, 0);
        return spList.getBoolean("isHeadTrack", true);
    }
}
