package com.baidu.cloud.videoplayer.widget;

import android.content.Context;
import android.graphics.Color;
import android.util.TypedValue;

import androidx.annotation.NonNull;

import com.baidu.cloud.videoplayer.demo.C;
import com.baidu.cloud.videoplayer.demo.info.DefaultDanmuParser;

import java.io.InputStream;
import java.util.HashMap;

import master.flame.danmaku.controller.DrawHandler;
import master.flame.danmaku.danmaku.loader.ILoader;
import master.flame.danmaku.danmaku.loader.IllegalDataException;
import master.flame.danmaku.danmaku.loader.android.DanmakuLoaderFactory;
import master.flame.danmaku.danmaku.model.BaseDanmaku;
import master.flame.danmaku.danmaku.model.DanmakuTimer;
import master.flame.danmaku.danmaku.model.IDisplayer;
import master.flame.danmaku.danmaku.model.android.DanmakuContext;
import master.flame.danmaku.danmaku.model.android.Danmakus;
import master.flame.danmaku.danmaku.parser.BaseDanmakuParser;
import master.flame.danmaku.danmaku.parser.IDataSource;
import master.flame.danmaku.ui.widget.DanmakuView;

public class DefaultDanmuView extends DanmakuView {

    private DanmakuContext mContext;
    private BaseDanmakuParser mParser;
    private long mInitStartPosMs = 0L;

    private static final float DEFAULT_SPEED = 1.2f;

    public DefaultDanmuView(@NonNull Context context, @NonNull InputStream inputStream) {
        super(context);
        mParser = createParser(context, inputStream);
        initDanmuView();
    }

    private void initDanmuView() {
        // 设置最大显示行数
        HashMap<Integer, Integer> maxLinesPair = new HashMap<>();
        maxLinesPair.put(BaseDanmaku.TYPE_SCROLL_RL, 5); // 滚动弹幕最大显示5行
        // 设置是否禁止重叠
        HashMap<Integer, Boolean> overlappingEnablePair = new HashMap<>();
        overlappingEnablePair.put(BaseDanmaku.TYPE_SCROLL_RL, true);
        overlappingEnablePair.put(BaseDanmaku.TYPE_FIX_TOP, true);

        mContext = DanmakuContext.create();
        mContext.setDanmakuStyle(IDisplayer.DANMAKU_STYLE_STROKEN, 3)
                .setDuplicateMergingEnabled(false)
                .setScrollSpeedFactor(DEFAULT_SPEED)
                .setScaleTextSize(1.2f)
                .setMaximumLines(maxLinesPair)
                .preventOverlapping(overlappingEnablePair)
                .setDanmakuMargin(40);
        setCallback(new DrawHandler.Callback() {
            @Override
            public void prepared() {
                if (mInitStartPosMs > 0) {
                    seekTo(mInitStartPosMs);
                }
                start();
            }

            @Override
            public void updateTimer(DanmakuTimer timer) {

            }

            @Override
            public void danmakuShown(BaseDanmaku danmaku) {

            }

            @Override
            public void drawingFinished() {

            }
        });
        enableDanmakuDrawingCache(true);
    }

    public void prepare() {
        prepare(0);
    }

    public void prepare(long initStartPosMs) {
        if (isPrepared()) {
            restart();
        }
        mInitStartPosMs = initStartPosMs;
        prepare(mParser, mContext);
    }

    @Override
    public void start() {
        if (isPrepared() && isPaused()) {
            super.resume();
        }
    }

    @Override
    public void pause() {
        if (isPrepared()) {
            super.pause();
        }
    }

    public void seekTo(long millisec) {
        if (isPrepared()) {
            super.seekTo(millisec);
        }
    }

    @Override
    public void clear() {
        super.clear();
        super.clearDanmakusOnScreen();
    }

    @Override
    public void release() {
        super.release();
    }

    @Override
    public void show() {
        super.show();
    }

    @Override
    public void hide() {
        super.hide();
    }

    public boolean isVisible() {
        return super.isShown();
    }

    public void setSpeed(float speed) {
        if (speed > 0) {
            mContext.setScrollSpeedFactor(DEFAULT_SPEED / speed);
        }
    }

    public void addDanmuItem(String text, boolean isManual) {
        BaseDanmaku danmaku = mContext.mDanmakuFactory.createDanmaku(BaseDanmaku.TYPE_SCROLL_RL);
        if (danmaku == null) {
            return;
        }

        danmaku.text = text;
        danmaku.padding = 5;
        // 手动发送的弹幕赋予高优先级，反之赋予低优先级，可能会被各种过滤器过滤并隐藏显示
        danmaku.priority = isManual ? (byte) 1 : 0;
        danmaku.isLive = false;
        danmaku.setTime(getCurrentTime() + 1200);
        danmaku.textSize = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, 12,
                getContext().getResources().getDisplayMetrics());
        danmaku.textColor = C.WHITE_80;
        danmaku.textShadowColor = Color.GRAY;
        danmaku.borderColor = isManual ? Color.GREEN : Color.TRANSPARENT;
        addDanmaku(danmaku);
    }


    private BaseDanmakuParser createParser(Context context, InputStream stream) {
        if (stream == null) {
            return new BaseDanmakuParser() {

                @Override
                protected Danmakus parse() {
                    return new Danmakus();
                }
            };
        }

        ILoader loader = DanmakuLoaderFactory.create(DanmakuLoaderFactory.TAG_BILI);

        try {
            loader.load(stream);
        } catch (IllegalDataException e) {
            e.printStackTrace();
        }
        BaseDanmakuParser parser = new DefaultDanmuParser(context);
        IDataSource<?> dataSource = loader.getDataSource();
        parser.load(dataSource);
        return parser;
    }


}
